$GameModeCycle::ConfigFile = "config/server/GAME_MODE_LIST.txt";
$GameModeCycle::PreLoaded = true;
$GameModeCycle::GameModeIndex = 0;
$GameModeCycle::Running = false;

$Pref::GameModeCycle::MinutesPerGameMode = 0;
$Pref::GameModeCycle::MiniGameRoundsPerGameMode = 2;

function gameModeCycleTick(%tick)
{
	cancel($GameModeCycle::tickSched);
	if(%tick >= $Pref::GameModeCycle::MinutesPerGameMode)
	{
		changeGameModeNext();
	}
	else
	{
		$GameModeCycle::tickSched = schedule(60000, 0, gameModeCycleTick, %tick ++);
	}
}

function changeGameModeNext()
{
	$GameModeCycle::GameModeIndex ++;
	if($GameModeCycle::GameModeIndex >= $GameModeCycle::GameModeCount)
		$GameModeCycle::GameModeIndex = 0;
	%gameMode = $GameModeCycle::GameMode[$GameModeCycle::GameModeIndex];
	if(%gameMode $= "Cycle")
		return;
	echo("CHANGE TO IDX" SPC $GameModeCycle::GameModeIndex @ ", MODE" SPC %gameMode);
	changeGameMode(%gameMode);
}

function importGameModeCycle(%path)
{
	if(!isFile(%path))
		return;
	$GameModeCycle::GameModeCount = 0;
	%file = new FileObject();
	%file.openForRead(%path);
	while(!%file.isEOF())
	{
		%gameMode = %file.readLine();
		if(isFile("Add-Ons/GameMode_" @ %gameMode @ "/GameMode.txt") &&
			%gameMode !$= "Cycle")
		{
			$GameModeCycle::GameMode[$GameModeCycle::GameModeCount] = %gameMode;
			$GameModeCycle::GameModeCount ++;
			echo("GameModeCycle added" SPC %gameMode);
		}
		else
		{
			error("GameModeCycle invalid game mode:" SPC %gameMode);
		}
	}
	%file.close();
	%file.delete();
}

function exportGameModeCycle(%path)
{
	if(!isWriteableFileName(%path))
		return;
	%file = new FileObject();
	%file.openForWrite(%path);
	for(%i = 0; %i < $GameModeCycle::GameModeCount; %i ++)
	{
		%file.writeLine($GameModeCycle::GameMode[%i]);
	}
	%file.close();
	%file.delete();
}

function dumpGameModeCycle()
{
	for(%i = 0; %i < $GameModeCycle::GameModeCount; %i ++)
	{
		%file.writeLine($GameModeCycle::GameMode[%i]);
	}
}

package GameModeCycle
{
	function MiniGameSO::reset(%this, %client)
	{
		if($GameModeCycle::Running)
		{
			if((!isFunction("isSlayerMiniGame") || !isSlayerMiniGame(%this)) &&
				%this.gmcRoundsPassed >= $Pref::GameModeCycle::MiniGameRoundsPerGameMode)
			{
				messageAll('', "Changing to next game mode.");
				schedule(5000, 0, changeGameModeNext);
				return;
			}
			else
			{
				%this.gmcRoundsPassed ++;
			}
		}
		parent::reset(%this, %client);
	}
	
	function Slayer_MiniGameSO::endRound(%this, %winner, %resetTime)
	{
		if($GameModeCycle::Running)
		{
			if(%this.gmcRoundsPassed >= $Pref::GameModeCycle::MiniGameRoundsPerGameMode)
			{
				messageAll('', "Changing to next game mode.");
				schedule(%this.timeBetweenRounds * 1000, 0, changeGameModeNext);
			}
			else
			{
				%this.gmcRoundsPassed ++;
			}
		}
		parent::endRound(%this, %winner, %resetTime);
	}

	function onMissionLoaded()
	{
		parent::onMissionLoaded();
		if($Pref::GameModeCycle::MinutesPerGameMode > 0)
		{
			gameModeCycleTick(0);
		}
	}
	
	function createServer(%type)
	{
		if(($Server::Dedicated && $gameModeArg $= "Cycle") ||
			$gameModeArg $= "Add-Ons/Gamemode_Cycle/GameMode.txt")
		{
			importGameModeCycle($GameModeCycle::ConfigFile);
			if($GameModeCycle::GameModeCount > 0)
			{
				%gameMode = $GameModeCycle::GameMode[0];
				%file = "Add-Ons/GameMode_" @ %gameMode @ "/GameMode.txt";
				if(isFile(%file))
				{
					echo("GameModeCycle is hijacking the loading process.");
					$GameModeCycle::Running = true;
					$gameModeArg = %file;
				}
			}
		}
		return parent::createServer(%type);
	}
	
	function deactivateServerPackages()
	{
		parent::deactivateServerPackages();
		activatePackage(GameModeCycle);
	}
	
	function onExit()
	{
		if($GameModeCycle::Running)
			exportGameModeCycle($GameModeCycle::ConfigFile);
		parent::onExit();
	}
};
activatePackage(GameModeCycle);